/*-----------------------------------------------------*/
/*--	Micro-EMPIX Display Drivers (Low)(DispLow.c)  -*/
/*--    Ver 1.0 (c) 2004                              -*/
/*--	E             -*/
/*--    ==================================            -*/
/*--    .                  -*/
/*--    .                        -*/
/*--    ..                       -*/
/*-----------------------------------------------------*/

#ifndef _DEFS_H
	#define _DEFS_H
	#include "defs.h"
#endif

/* ====================================================*/
/* ============== Function Prototypes =================*/
/* ====================================================*/

void PrintChar (int x, int y, char content);

void bSetColor(SCR_COLORS fgcolor, SCR_COLORS bgcolor, unsigned int HighIntensity);
void bSetCursor(int x, int y);
void bGetCursor(int *x, int *y);
void bcls(void);
void bAdvanceCursor(void);
void bBacktrackCursor(void);
void bNewLine(void);
void bPrintChar(char c);
void bScrollWindow(const char UpperLeftX, const char UpperLeftY, const char LowerRightX, const char LowerRightY, const char UpDown);


unsigned char SCR_Current_Color; 	/*       */
									/*      */
									/*   BIOS software interrupts */

/*==========================================================================================*/
/*=====================  O (MEMORY MAPPED) =====================================*/
/*==========================================================================================*/

/* void PrintChar (int x, int y, char content):    */
/* int x, int y:     x,y */
/* char content:     */

void PrintChar (int x, int y, char content)
{
	int pos;
	pos=y*160+2*x;
	asm mov bx,pos;
	asm mov ax,0B800h ;
	asm mov es,ax;
	asm mov ah,00Fh ;
	asm mov al, content;
	asm mov es: [bx],ax ;
}



/*==========================================================================================*/
/*==============================  O (BIOS) =====================================*/
/*==========================================================================================*/

/*                 */
/*       BIOS  .  High Intensity   */
/*   flag <> 0 */

void bSetColor(SCR_COLORS fgcolor, SCR_COLORS bgcolor, unsigned int HighIntensity)
{
        /* Disable interrupts because someone may try to read it inside an interrupt handler
        in an unsafe state (although not recommended) */
        asm PUSHF
        asm CLI
        SCR_Current_Color = ((bgcolor << 4) | fgcolor) & 0xFF;
        if(HighIntensity)
        	SCR_Current_Color |= 0x8;
        asm POPF
}

/*   cursor        10
/*   2 */

void bSetCursor(int x, int y)
{
        asm MOV AH,2
        asm MOV DH,y
        asm MOV DL,x
        asm MOV BH,0
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
}

/*       cursor  ,    */
/*  10   3 */

void bGetCursor(int *x, int *y)
{
        if(x && y)
        {
			asm MOV AH,3
            asm MOV BH,0
#if SIRQS_ISOLATED == 1
			asm PUSHF
			asm CLI
#endif
            asm INT 10H
#if SIRQS_ISOLATED == 1
			asm POPF
#endif

            *x = _DL;
            *y = _DH;
        }
}

/*       10   6 */

void bcls(void)
{
        asm MOV AH,6
        asm MOV AL,0
        asm MOV CH,0
        asm MOV CL,0
        asm MOV DH,24
        asm MOV DL,79
        asm MOV BH,07H /*   default  */
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
        /*      0,0 */
        asm MOV AH,2
        asm XOR DX,DX
        asm XOR BH,BH
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
}

/*   cursor     .   cursor      (column 79) */
/*         .   cursor     ,  */
/*   scroll                */
/*     */

void bAdvanceCursor(void)
{
		/*  : 25 X 80 */
        asm MOV AH,3
        asm MOV BH,0
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif

        asm MOV AH,2
        asm INC DL
        /*    */
        asm CMP DL,80
        asm JB cont1
        asm XOR DL,DL
        asm INC DH
        /*    */
        asm CMP DH,25
        asm JB cont1
        /* scroll up    */
        asm PUSH AX
        asm PUSH DX
        asm MOV AH,6
        asm MOV AL,1
        asm MOV CH,0
        asm MOV CL,0
        asm MOV DH,24
        asm MOV DL,79
        asm MOV BH,SCR_Current_Color
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
        asm POP DX
        asm POP AX
        asm DEC DH
        asm XOR BH,BH
cont1:
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
		asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
}

/*    ,     cursor */

void bBacktrackCursor(void)
{

		/*  : 25 X 80 */
        asm MOV AH,3
        asm MOV BH,0

#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif

        asm MOV AH,2
        /*    */
        asm CMP DL,0
        asm JG cont1
        asm MOV DL,80
        asm CMP DH,0
        asm JG cont2
        /* scroll down    */
        asm PUSH AX
        asm PUSH DX
        asm MOV AH,6
        asm MOV AL,2
        asm MOV CH,0
        asm MOV CL,0
        asm MOV DH,24
        asm MOV DL,79
        asm MOV BH,SCR_Current_Color

#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif

        asm POP DX
        asm POP AX
        asm XOR BH,BH
cont2:  asm DEC DH
cont1:  asm DEC DL

#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
		asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
}

/*  .          scroll */
/*           cursor      */
/*   */

void bNewLine(void)
{
        asm MOV AH,3
        asm MOV BH,0
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif

        asm MOV AH,2
        asm INC DH
        /*    */
        asm CMP DH,25
        asm JB cont1
        /* scroll up    */
        asm PUSH AX
        asm PUSH DX
        asm MOV AH,6
        asm MOV AL,1
        asm MOV CH,0
        asm MOV CL,0
        asm MOV DH,24
        asm MOV DL,79
        asm MOV BH,SCR_Current_Color
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
        asm POP DX
        asm POP AX
        asm DEC DH
cont1:  asm XOR DL,DL
        asm XOR BH,BH
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
}

/*    c       cursor      */
/*        (    AdvanceCursor).   */
/*   (   attribute)       bSetColor */

void bPrintChar(char c)
{
        /*   */
        asm MOV AH,9h
        asm MOV AL,c
        asm MOV BH,0H
        asm MOV BL,SCR_Current_Color
        asm MOV CX,1
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif

        asm MOV AH,3
        asm MOV BH,0
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif

        asm MOV AH,2
        asm INC DL
        /*    */
        asm CMP DL,80
        asm JB cont1
        asm XOR DL,DL
        asm INC DH
        /*    */
        asm CMP DH,25
        asm JB cont1
        /* scroll up    */
        asm PUSH AX
        asm PUSH DX
        asm MOV AH,6
        asm MOV AL,1
        asm MOV CH,0
        asm MOV CL,0
        asm MOV DH,24
        asm MOV DL,79
        asm MOV BH,SCR_Current_Color
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
        asm POP DX
        asm POP AX
        asm DEC DH
        asm XOR BH,BH
cont1:
#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
		asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
}



/* UpDown = 0, means down, otherwise means up */
/* Scrolling         (UpperLeftX, UpperLeftY) & (LowerRightX, LowerRightY). */
/*    -       (0,0) */
/* UpDown = 0, means down, otherwise means up */

void bScrollWindow(const char UpperLeftX, const char UpperLeftY, const char LowerRightX, const char LowerRightY, const char UpDown)
{
        /* scroll window up */
        if(UpDown)
			asm MOV AH,6
        else
            asm MOV AH,7
        asm MOV AL,1
        asm MOV CH,BYTE PTR UpperLeftX
        asm MOV CL,BYTE PTR UpperLeftY
        asm MOV DH,BYTE PTR LowerRightX
        asm MOV DL,BYTE PTR LowerRightY
        asm MOV BH,07H

#if SIRQS_ISOLATED == 1
		asm PUSHF
		asm CLI
#endif
        asm INT 10H
#if SIRQS_ISOLATED == 1
		asm POPF
#endif
}