/*-----------------------------------------------------*/
/*--	Micro-EMPIX UART Drivers (Low)(UARTHigh.c)    -*/
/*--    Ver 1.0 (c) 2004                              -*/
/*--	E             -*/
/*--    ==================================            -*/
/*--    .                  -*/
/*--    .                        -*/
/*--    ..                       -*/
/*-----------------------------------------------------*/

#ifndef _DEFS_H
	#define _DEFS_H
	#include "defs.h"
#endif

#ifndef _UARTLOW_H
	#define _UARTLOW_H
	#include "UARTLow.h"
#endif

#ifndef _SEM_H
	#define _SEM_H
	#include "sem.h"
#endif


/* ====================================================*/
/* ============== Function Prototypes =================*/
/* ====================================================*/
int Distance(char *grt, char *lst);
void Handler(SER_PORT port, char *InBuf, char **SER_InBufPtrWr, char **SER_InBufPtrRd, int ISEM, int OSEM);
void Handler1(void);
void Handler2(void);
void QSR_InitializePort(SER_PORT portnum, BAUD_RATE brate);
char QSR_ReadChar(SER_PORT portnum);
int QSR_WriteChar(SER_PORT portnum, char ch);


struct QSR_ERROR QSR_Error;


/*  buffer         */
static char SER1_InBuf[BUFLEN];
static char *SER1_InBufPtrRd, *SER1_InBufPtrWr;

/*  buffer         */
static char SER2_InBuf[BUFLEN];
static char *SER2_InBufPtrRd, *SER2_InBufPtrWr;

/*     buffers */
int Distance(char *grt, char *lst)
{
        return (int)(grt >= lst ? grt - lst : BUFLEN - (lst - grt));
}

/*       interrupt handlers      */
void Handler(SER_PORT port, char *InBuf, char **SER_InBufPtrWr, char **SER_InBufPtrRd, int ISEM, int OSEM)
{
	char buf[16];
	int i, ans;
	/* Find out what happened */
    ans = InterruptReason(port);
    switch(ans)
	{
		case 1: /* new char available */
			ans = SER_ReadChar(port, buf);
			if(ans < 0)
			{
				QSR_Error.Error_Type = 1;
				QSR_Error.Error = ans;
                                return;
			}
			else
				QSR_Error.Error_Type = 0;

			for(i = 0; i < ans; i++)
			{
                 if(Distance(*SER_InBufPtrWr, *SER_InBufPtrRd) >= BUFLEN - 1)
                 {
					 /* If we have exceeded our buffer capacity, discard the next chars */
                     QSR_Error.Error_Type = 3;
                     return;
                  }
				  **SER_InBufPtrWr = buf[i];
                  (*SER_InBufPtrWr)++;
                  if(*SER_InBufPtrWr - InBuf >= BUFLEN)
				  {
					*SER_InBufPtrWr = InBuf;
					*InBuf = buf[i];
				  }
                  V(ISEM);
            }
			break;
        case 2:
                  V(OSEM); /* Give Permission to write */
			break;
		default:
			break;
	}
}

void Handler1(void)
{
        Handler(SER_P1, SER1_InBuf, &SER1_InBufPtrWr, &SER1_InBufPtrRd, ISEM1, OSEM1);
}

void Handler2(void)
{
        Handler(SER_P2, SER2_InBuf, &SER2_InBufPtrWr, &SER2_InBufPtrRd, ISEM2, OSEM2);
}

void QSR_InitializePort(SER_PORT portnum, BAUD_RATE brate)
{
	struct IrqConf mconf;
	int i;

	if(portnum == SER_P1)
		mconf.IrqActions = Handler1;
        else
		mconf.IrqActions = Handler2;

	/* trigger interrupts when a new char is ready or when a character can be sent */
        mconf.IR_Byte = IRQ_When_Char_Ready | IRQ_When_THR_Empty;
	/* trigger interrupt with every character coming */
	mconf.Chars_Bef_Irq = ch1;

    asm PUSHF
    asm CLI
    SER_InitializePort(portnum, brate, 0, &mconf);

	if(portnum == SER_P1)
	{
        SER1_InBufPtrRd = SER1_InBuf;
        SER1_InBufPtrWr = SER1_InBuf;
		V(OSEM1);
	}
	else
	{
        SER2_InBufPtrRd = SER2_InBuf;
        SER2_InBufPtrWr = SER2_InBuf;
		V(OSEM2);
	}
        asm POPF
}


char QSR_ReadChar(SER_PORT portnum)
{
	char ch = 0;
	if(portnum == SER_P1)
	{
		/*           */
		/*   buffer  SER1_InBufPtrRd */
        P(ISEM1);
		ch = *SER1_InBufPtrRd++;
		if(SER1_InBufPtrRd - SER1_InBuf >= BUFLEN)
			SER1_InBufPtrRd = SER1_InBuf;
	}
	else
	{
		/*           */
		/*   buffer  SER2_InBufPtrRd */
        P(ISEM2);
		ch = *SER2_InBufPtrRd++;
		if(SER2_InBufPtrRd - SER2_InBuf >= BUFLEN)
			SER2_InBufPtrRd = SER2_InBuf;
	}
	return ch;
}

int QSR_WriteChar(SER_PORT portnum, char ch)
{
	int ans;
    /* Block for permission from the interrupt handler */
	/*      UART     */
	/*      .    */
	/*    interrupt handler */
	if(portnum == SER_P1)
		P(OSEM1);
	else if(portnum == SER_P2)
		P(OSEM2);
	else /* should not happen */
		return 10;

	ans = SER_WriteChar(portnum, ch);
	return ans;
}
